
local scrutinizer = {}
_G["scrutinizer"] = scrutinizer


local mceil = math.ceil
local mfloor = math.floor
local pairs = pairs
local ipairs = ipairs
local tinsert = table.insert
local tsort = table.sort
local unpack = unpack
local tonumber = tonumber
local tostring = tostring
local stringfind = string.find
local dofile = dofile
local stringformat = string.format
local stringsub = string.sub
local stringupper = string.upper
local stringlower = string.lower


local addOnName = "scrutinizer"
local addOnVersion = "2.4"
local addOnAuthor = "NashaRiochtDE"

local addOnLoaded = false

local oocTrigger = 5
local maxPageNum = 1

local languages	= {"DE", "JP"}	--"FR", "ES", "CN", "TW", "JP", "RU"
dofile("Interface/AddOns/scrutinizer/lang/EN.lua")
for _, lang in ipairs(languages) do
	if stringfind(GetLanguage(), lang) then
		dofile("Interface/AddOns/scrutinizer/lang/"..lang..".lua")
		break
	end
end

local cats = SCRUTINIZER_CATS

local skillTypes = {"NORMAL", "CRITIAL", "DODGE", "PARRY", "MISS", "RESIST", "DOUBLE", "HALF"}

local classColors = {
	["WARRIOR"] = {1, 0, 0},
	["RANGER"] = {0, 1, 0},
	["THIEF"] = {0, 1, 1},
	["MAGE"] = {1, 0.75, 0},
	["AUGUR"] = {0, 0, 1},
	["KNIGHT"] = {1, 1, 0},
	["DRUID"] = {0, 0, 0},
	["RUNEDANCER"] = {0, 0, 0},
	["GM"] = {0, 0, 0},
}

local shrinkOrder = {
	["TOP"] = {
		["scrutinizer_Main_TitleFrame"] = {"TOPLEFT", "TOPLEFT", "scrutinizer_MainFrame", 0, 0},
		["scrutinizer_Main_SelfFrame"] = {"TOPLEFT", "BOTTOMLEFT", "scrutinizer_Main_TitleFrame", 0, 0},
		["scrutinizer_Main_BarFrame"] = {"TOPLEFT", "BOTTOMLEFT", "scrutinizer_Main_SelfFrame", 0, 0},
		["scrutinizer_Main_PartyFrame"] = {"TOPLEFT", "BOTTOMLEFT", "scrutinizer_Main_BarFrame", 0, 0},
	},
	["BOTTOM"] = {
		["scrutinizer_Main_TitleFrame"] = {"BOTTOMLEFT", "BOTTOMLEFT", "scrutinizer_MainFrame", 0, 0},
		["scrutinizer_Main_SelfFrame"] = {"BOTTOMLEFT", "TOPLEFT", "scrutinizer_Main_TitleFrame", 0, 0},
		["scrutinizer_Main_PartyFrame"] = {"BOTTOMLEFT", "TOPLEFT", "scrutinizer_Main_SelfFrame", 0, 0},
		["scrutinizer_Main_BarFrame"] = {"BOTTOMLEFT", "TOPLEFT", "scrutinizer_Main_PartyFrame", 0, 0},
	},
}


scrutinizer_onUpdateTimer = 0
scrutinizer_onUpdateRepeatTimer = 0.8


function scrutinizer_OnLoad(this)
	this:RegisterEvent("VARIABLES_LOADED")
	this:RegisterEvent("PARTY_MEMBER_CHANGED")
	this:RegisterEvent("COMBATMETER_DAMAGE")
	this:RegisterEvent("COMBATMETER_HEAL")
	this:RegisterEvent("SAVE_VARIABLES")
	SlashCmdList["scrutinizer"] = scrutinizer_SlashCmdHandler
	_G["SLASH_scrutinizer1"] = "/scrut"
	SaveVariables("scrutinizerSave")
end


function scrutinizer_OnEvent(event)
	if event == "VARIABLES_LOADED" then
		scrutinizer = scrutinizerSaveData
		DEFAULT_CHAT_FRAME:AddMessage(addOnName.."ロード完了。コマンドリストは'/scrut'で表示されます")
		return
	end
	if event == "PARTY_MEMBER_CHANGED" then
		if arg1 and addOnLoaded == true and playerName == UnitName("player") then
			if scrutinizer[playerName]["cfg"]["configframe"]["resetOnPartyJoin"] == true then
				if stringfind(arg1, playerName) then
					scrutinizer_ResetDB()
				end
			end
		end
		return
	end	
	if event == "COMBATMETER_DAMAGE" then
		local tbl = {_source, _skill, _type, _damage, GetTime()}
		if tbl[1] == playerName or InPartyByName(tbl[1]) == true or InRaidByName(tbl[1]) == true then
			scrutinizer_TblMod(cats[1], tbl)
		end
		local tbl = {_target, _skill, _type, _damage, GetTime()}
		if tbl[1] == playerName or InPartyByName(tbl[1]) == true or InRaidByName(tbl[1]) == true then
			scrutinizer_TblMod(cats[2], tbl)
		end
		return
	end
	if event == "COMBATMETER_HEAL" then
		local tbl = {_source, _skill, _type, _heal, GetTime()}
		if tbl[1] == playerName or InPartyByName(tbl[1]) == true or InRaidByName(tbl[1]) == true then
			scrutinizer_TblMod(cats[3], tbl)
		end
		local tbl = {_target, _skill, _type, _heal, GetTime()}
		if tbl[1] == playerName or InPartyByName(tbl[1]) == true or InRaidByName(tbl[1]) == true then
			scrutinizer_TblMod(cats[4], tbl)
		end
		return
	end
	if event == "SAVE_VARIABLES" then
		local x, y = scrutinizer_MainFrame:GetPos()
		local scaleFactor = scrutinizer_GetScaleFactor()
		scrutinizer[playerName]["cfg"] = {
			["mainframe"] = {
				["x"] = x,
				["y"] = y,
				["cat"] = scrutinizer_Main_TitleFrame_Cat:GetText(),
			},
			["detailframe"] = {},
			["configframe"] = {
				["showParty"] = scrutinizer[playerName]["cfg"]["configframe"]["showParty"],
				["showPartyInParty"] = scrutinizer[playerName]["cfg"]["configframe"]["showPartyInParty"],
				["shrinkTo"] = scrutinizer[playerName]["cfg"]["configframe"]["shrinkTo"],
				["pinned"] = scrutinizer[playerName]["cfg"]["configframe"]["pinned"],
				["scale"] = scaleFactor,
				["resetOnPartyJoin"] = scrutinizer[playerName]["cfg"]["configframe"]["resetOnPartyJoin"],
			},
		}
		
		scrutinizerSaveData = scrutinizer
	end
end


--------------------------onupdate functions
function scrutinizer_OnUpdate()

	if addOnLoaded == true then
		if not scrutinizer[playerName] then
			addOnLoaded = false
			return
		end
		
		if scrutinizer[playerName]["cfg"]["configframe"]["showPartyInParty"] == true then
			if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
				if scrutinizer_Main_BarFrame:IsVisible() == false then
					scrutinizer_Main_BarFrame:Show()
					scrutinizer_Main_PartyFrame:Show()
				end
			else
				if scrutinizer_Main_BarFrame:IsVisible() == true then
					scrutinizer_Main_BarFrame:Hide()
					scrutinizer_Main_PartyFrame:Hide()
				end
			end
		end
		
		local _, catName = scrutinizer_GetCat()
		scrutinizer_DoMain(catName)
		return
	end
	if addOnLoaded == false then
		if not playerName then
			playerName = ""
		else
			playerName = UnitName("player")
		end
		if playerName == nil or playerName == "" then
			return
		end
		if scrutinizer == nil then
			scrutinizer = {}
		end
		if not scrutinizer[playerName] then
			scrutinizer[playerName] = {}
			scrutinizer[playerName] = {
				["db"] = {},
				["cfg"] = {
					["mainframe"] = {
						["x"] = 0,
						["y"] = 0,
						["cat"] = cats[1],
					},
					["detailframe"] = {},
					["configframe"] = {
						["showParty"] = true,
						["showPartyInParty"] = false,
						["shrinkTo"] = "TOP",
						["pinned"] = false,
						["scale"] = 1,
						["resetOnPartyJoin"] = false,
					},
				},
			}
			scrutinizer_ResetDB()
		end
		
		if not scrutinizer[playerName]["cfg"]["configframe"]["resetOnPartyJoin"] then
			scrutinizer[playerName]["cfg"]["configframe"]["resetOnPartyJoin"] = false
		end
		
		local uiScale, x, y = GetUIScale(), scrutinizer[playerName]["cfg"]["mainframe"]["x"], scrutinizer[playerName]["cfg"]["mainframe"]["y"]
		--set mainframe position
		scrutinizer_MainFrame:ClearAllAnchors()
		local scrut_scaleFactor = scrutinizer[playerName]["cfg"]["configframe"]["scale"]
		scrutinizer_MainFrame:SetAnchor("TOPLEFT", "TOPLEFT", "UIParent", (x/uiScale)/scrut_scaleFactor, (y/uiScale)/scrut_scaleFactor)
		--set std cat
		scrutinizer_Main_TitleFrame_Cat:SetText(scrutinizer[playerName]["cfg"]["mainframe"]["cat"])
		--set std page
		scrutinizer_Main_PartyFrame_BarPage:SetText("1")
		--set additional
		scrutinizer_Main_SelfFrame_F1:SetText(playerName)
		scrutinizer_Main_SelfFrame_AttF1:SetText(SCRUTINIZER_TXT_NAME)
		scrutinizer_Main_SelfFrame_AttF2:SetText(SCRUTINIZER_TXT_VALUEPERCENT)
		scrutinizer_Main_SelfFrame_AttF3:SetText(SCRUTINIZER_TXT_CPSTPS)
		--set config
		scrutinizer_ConfigFrame_Title_Text1:SetText(addOnName..":")
		scrutinizer_ConfigFrame_Title_Text2:SetText(SCRUTINIZER_TT_CONFIG)
		
		scrutinizer_ConfigFrame_CB_showParty_Text:SetText(SCRUTINIZER_TXT_SHOWPARTY)
		if scrutinizer[playerName]["cfg"]["configframe"]["showParty"] == true then
			scrutinizer_Main_BarFrame:Show()
			scrutinizer_Main_PartyFrame:Show()
		else
			scrutinizer_Main_BarFrame:Hide()
			scrutinizer_Main_PartyFrame:Hide()
		end
		
		scrutinizer_ConfigFrame_CB_showPartyInParty_Text:SetText(SCRUTINIZER_TXT_SHOWPARTYINPARTY)
		
		scrutinizer_ConfigFrame_CB_resetOnPartyJoin_Text:SetText(SCRUTINIZER_TXT_RESETONPARTYJOIN)
		
		scrutinizer_ConfigFrame_CB_pinned_Text:SetText(SCRUTINIZER_TXT_PINNED)
		if scrutinizer[playerName]["cfg"]["configframe"]["pinned"] == true then
			scrutinizer_Main_TitleFrameMoveBtn:Hide()
			scrutinizer_Main_TitleFrameCloseBtn:Hide()
		end
		--set shrink
		scrutinizer_ConfigFrame_DropDown_shrinkDir_Text:SetText(SCRUTINIZER_TXT_SHRINKDIR)
		scrutinizer_shrinkItTo(scrutinizer[playerName]["cfg"]["configframe"]["shrinkTo"])
		scrutinizer_ConfigFrame_DropDown_shrinkDir_OnLoad()
		--set scale
		scrutinizer_ConfigFrame_Slider_ScaleDesc:SetText(SCRUTINIZER_TXT_SLIDERSCALE)
		scrutinizer_SetScaleFactor(scrutinizer[playerName]["cfg"]["configframe"]["scale"])
		--reset position button text
		scrutinizer_ConfigFrame_BTN_reset_position_Text:SetText(SCRUTINIZER_TXT_RESETPOS)
		--set whisperTo
		scrutinizer_ConfigFrame_DropDown_whisperTo_Text:SetText(SCRUTINIZER_TXT_WHISPERTO)
		scrutinizer_ConfigFrame_DropDown_whisperTo_OnLoad()
		
		--detailframe set atts
		scrutinizer_DetailFrame_Body_Att_F1:SetText(SCRUTINIZER_TXT_NAME)
		scrutinizer_DetailFrame_Body_Att_F2:SetText(SCRUTINIZER_TXT_VALUEPERCENT)
		scrutinizer_DetailFrame_Body_Att_F3:SetText(SCRUTINIZER_TXT_COUNTER)
		scrutinizer_DetailFrame_Body_Att_F4:SetText(SCRUTINIZER_TXT_AVERAGE)
		
		scrutinizer_DetailFrame_Body_Att_F5:SetText(SCRUTINIZER_TXT_NORMAL)
		scrutinizer_DetailFrame_Body_Att_F6:SetText(SCRUTINIZER_TXT_CRITICAL)
		scrutinizer_DetailFrame_Body_Att_F7:SetText(SCRUTINIZER_TXT_DOUBLE)
		scrutinizer_DetailFrame_Body_Att_F8:SetText(SCRUTINIZER_TXT_MISS)
		scrutinizer_DetailFrame_Body_Att_F9:SetText(SCRUTINIZER_TXT_RESIST)
		scrutinizer_DetailFrame_Body_Att_F10:SetText(SCRUTINIZER_TXT_DODGE)
		scrutinizer_DetailFrame_Body_Att_F11:SetText(SCRUTINIZER_TXT_PARRY)
		
		addOnLoaded = true
	end
end

function scrutinizer_DoMain(currentCat)
	if scrutinizer[playerName] and scrutinizer_MainFrame:IsVisible() then
		local sortTbl, highestValue = {}, 0
		local raidValue, raidLastPs, raidPs = 0, 0, 0
		local mainTbl = scrutinizer[playerName]["db"][currentCat]
		for k, v in pairs(mainTbl) do
			local name, value = k, mainTbl[k]["value"]
			local totalPsTime = mainTbl[k]["totalPsTime"]
			if name == playerName or InPartyByName(name) == true or InRaidByName(name) == true then
				tinsert(sortTbl, {
					name = name,
					value = value,
				})
				raidValue = raidValue + value
			end
		end
		local sortTblNum = #sortTbl
		scrutinizer_SetMaxPage(sortTblNum)
		--if table empty return
		if sortTblNum == 0 then
			scrutinizer_mainframe_clean()
			return
		end
		--sort by value
		tsort(sortTbl, function(a, b)
			return (a["value"] > b["value"])
		end)
		--set highest value
		if sortTbl[1] then
			highestValue = sortTbl[1]["value"]
		end
		if sortTbl then
			--set starter depending on current page
			local startCounter = ((6 * scrutinizer_GetPage()) - 5)
			local barCounter = 1
			scrutinizer_mainframe_clean()
			for k = 1, sortTblNum, 1 do
				local tbl = mainTbl[sortTbl[k]["name"]]
				--set self values
				if sortTbl[k]["name"] == playerName then
					scrutinizer_Main_SelfFrame_F2:SetText(tbl["value"])
					scrutinizer_Main_SelfFrame_F3:SetText(scrutinizer_DivRound(0, tbl["lastFightValue"], tbl["lastPsTime"] - tbl["startPsTime"]).."/"..scrutinizer_DivRound(0, tbl["value"], tbl["totalPsTime"]))
				end
				if (GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0) and scrutinizer_Main_BarFrame:IsVisible() and sortTblNum > 0 then
					if k >= startCounter then
						if sortTbl[k]["name"] == playerName then
							scrutinizer_Main_SelfFrame_F2:SetText(tbl["value"].."("..scrutinizer_DivRound(1, tbl["value"], raidValue)..")")
						end
						--set bar values
						_G["scrutinizer_Main_BarFrame_"..barCounter.."F1"]:SetText(sortTbl[k]["name"])
						_G["scrutinizer_Main_BarFrame_"..barCounter.."F2"]:SetText(tbl["value"].."("..scrutinizer_DivRound(1, tbl["value"], raidValue)..")")
						local playerLastPs = scrutinizer_DivRound(0, tbl["lastFightValue"], tbl["lastPsTime"] - tbl["startPsTime"])
						local playerTotalPs = scrutinizer_DivRound(0, tbl["value"], tbl["totalPsTime"])
						_G["scrutinizer_Main_BarFrame_"..barCounter.."F3"]:SetText(playerLastPs.."/"..playerTotalPs)
						--set party/raid values
						_G["scrutinizer_Main_PartyFrame_F1"]:SetText(raidValue)
						raidLastPs = raidLastPs + playerLastPs
						raidPs = raidPs + playerTotalPs
						_G["scrutinizer_Main_PartyFrame_F2"]:SetText(raidLastPs.."/"..raidPs)
						--set barwidth
						_G["scrutinizer_Main_BarFrame_Bar"..barCounter]:SetWidth((242/100)*scrutinizer_DivRound(1, tbl["value"], highestValue))
						--set barcolor by class
						local r, g, b = 0, 0, 0
						if playerName == sortTbl[k]["name"] then
							for j, _ in pairs(classColors) do
								if j == UnitClassToken("player") then
									r, g, b = unpack(classColors[j])
									break
								end
							end
						else	
							for i = 1, 36, 1 do
								if UnitName("raid"..i) == sortTbl[k]["name"] then
									for j, _ in pairs(classColors) do
										if j == UnitClassToken("raid"..i) then
											r, g, b = unpack(classColors[j])
											break
										end
									end
								end
							end
						end
						_G["scrutinizer_Main_BarFrame_Bar"..barCounter]:SetColor(r, g, b, 1)
					end
				end
				if barCounter == 6 then
					return
				end
				barCounter = barCounter + 1
			end
		end
	end
end


--------------------------event table modification functions
function scrutinizer_GetSkillType(skillType)
	local tbl = {}
	for k = 1, #skillTypes, 1 do
		tbl[k] = 0
		if skillType == skillTypes[k] then
			tbl[k] = 1
		end
	end
	return tbl
end

function scrutinizer_TblMod(catToFill, valueTbl)
	local tbl = scrutinizer[playerName]["db"][catToFill]
	local sourceORtarget, skill, skillType, value, eventTime = unpack(valueTbl)

	local normal, critical, dodge, parry, miss, resist, double, half = unpack(scrutinizer_GetSkillType(skillType))
	
	if skill == "ATTACK" and double == 1 then
		double = 0
		critical = 1
	end
	if half == 1 then
		half = 0
		parry = 1
	end
	--Ps
	if not tbl[sourceORtarget] then
		tbl[sourceORtarget] = {}
		tbl[sourceORtarget] = {
			["value"] = value,
			["lastFightValue"] = value,
			["startPsTime"] = eventTime,
			["lastPsTime"] = eventTime,
			["totalPsTime"] = 0,
			["skills"] = {},
		}
	else
		local tbl = scrutinizer[playerName]["db"][catToFill][sourceORtarget]
		local diff = eventTime - tbl["lastPsTime"]
		if diff > oocTrigger then
			tbl["lastFightValue"] = 0
			tbl["startPsTime"] = eventTime
		else
			tbl["totalPsTime"] = tbl["totalPsTime"] + diff
		end
		tbl["value"] = tbl["value"] + value
		tbl["lastFightValue"] = tbl["lastFightValue"] + value
		tbl["lastPsTime"] = eventTime
	end
	--skills
	local tbl = scrutinizer[playerName]["db"][catToFill][sourceORtarget]["skills"]
	if not tbl[skill] then
		tbl[skill] = {
			["value"] = value,
			["normal"] = normal,
			["critical"] = critical,
			["dodge"] = dodge,
			["parry"] = parry,
			["miss"] = miss,
			["resist"] = resist,
			["double"] = double,
		}
	else
		tbl[skill]["value"] = tbl[skill]["value"] + value
		tbl[skill]["normal"] = tbl[skill]["normal"] + normal
		tbl[skill]["critical"] = tbl[skill]["critical"] + critical
		tbl[skill]["dodge"] = tbl[skill]["dodge"] + dodge
		tbl[skill]["parry"] = tbl[skill]["parry"] + parry
		tbl[skill]["miss"] = tbl[skill]["miss"] + miss
		tbl[skill]["resist"] = tbl[skill]["resist"] + resist
		tbl[skill]["double"] = tbl[skill]["double"] + double
	end
end


--------------------------main frame button functions
function scrutinizer_mainframe_prevCat()
	local theCat, _ = scrutinizer_GetCat()
	if theCat <= 1 then
		theCat = #cats
	else
		theCat = theCat - 1
	end
	scrutinizer_Main_TitleFrame_Cat:SetText(cats[theCat])
end

function scrutinizer_mainframe_nextCat()
	local theCat, _ = scrutinizer_GetCat()
	if theCat >= #cats then
		theCat = 1
	else
		theCat = theCat + 1
	end
	scrutinizer_Main_TitleFrame_Cat:SetText(cats[theCat])
end

function scrutinizer_mainframe_prevPage()
	local thePage = scrutinizer_GetPage()
	if thePage > 1 then
		thePage = thePage - 1
	end
	thePage = tostring(thePage)
	scrutinizer_Main_PartyFrame_BarPage:SetText(thePage)
end

function scrutinizer_mainframe_nextPage()
	local thePage = scrutinizer_GetPage()
	if thePage < maxPageNum then
		thePage = thePage + 1
	end
	thePage = tostring(thePage)
	scrutinizer_Main_PartyFrame_BarPage:SetText(thePage)
end


--------------------------config frame functions
function scrutinizer_cfg_showParty_OnShow(this)
	if scrutinizer_Main_BarFrame:IsVisible() then
		this:SetChecked(scrutinizer[playerName]["cfg"]["configframe"]["showParty"])
	end
end

function scrutinizer_cfg_showParty_OnClick(this)
	if this:IsChecked() then
		scrutinizer_ConfigFrame_CB_showPartyInParty:SetChecked(false)
		scrutinizer[playerName]["cfg"]["configframe"]["showPartyInParty"] = false
		scrutinizer[playerName]["cfg"]["configframe"]["showParty"] = true
		scrutinizer_Main_BarFrame:Show()
		scrutinizer_Main_PartyFrame:Show()
	else
		scrutinizer[playerName]["cfg"]["configframe"]["showParty"] = false
		scrutinizer_Main_BarFrame:Hide()
		scrutinizer_Main_PartyFrame:Hide()
	end
end

function scrutinizer_cfg_showPartyInParty_OnShow(this)
	this:SetChecked(scrutinizer[playerName]["cfg"]["configframe"]["showPartyInParty"])
end

function scrutinizer_cfg_showPartyInParty_OnClick(this)
	if this:IsChecked() then
		scrutinizer_ConfigFrame_CB_showParty:SetChecked(false)
		scrutinizer[playerName]["cfg"]["configframe"]["showParty"] = false
		scrutinizer[playerName]["cfg"]["configframe"]["showPartyInParty"] = true
	else
		scrutinizer[playerName]["cfg"]["configframe"]["showPartyInParty"] = false
		scrutinizer_Main_BarFrame:Hide()
		scrutinizer_Main_PartyFrame:Hide()
	end
end

function scrutinizer_cfg_resetOnPartyJoin_OnShow(this)
	this:SetChecked(scrutinizer[playerName]["cfg"]["configframe"]["resetOnPartyJoin"])
end

function scrutinizer_cfg_resetOnPartyJoin_OnClick(this)
	if this:IsChecked() then
		scrutinizer[playerName]["cfg"]["configframe"]["resetOnPartyJoin"] = true
	else
		scrutinizer[playerName]["cfg"]["configframe"]["resetOnPartyJoin"] = false
	end
end

function scrutinizer_cfg_pinned_OnShow(this)
	this:SetChecked(scrutinizer[playerName]["cfg"]["configframe"]["pinned"])
end

function scrutinizer_cfg_pinned_OnClick(this)
	if scrutinizer_Main_TitleFrameMoveBtn:IsVisible() then
		scrutinizer[playerName]["cfg"]["configframe"]["pinned"] = true
		scrutinizer_Main_TitleFrameMoveBtn:Hide()
		scrutinizer_Main_TitleFrameCloseBtn:Hide()
	else
		scrutinizer[playerName]["cfg"]["configframe"]["pinned"] = false
		scrutinizer_Main_TitleFrameMoveBtn:Show()
		scrutinizer_Main_TitleFrameCloseBtn:Show()
	end
end

function scrutinizer_ConfigFrame_DropDown_shrinkDir_OnLoad()
	UIDropDownMenu_Initialize(scrutinizer_ConfigFrame_DropDown_shrinkDir, scrutinizer_ConfigFrame_DropDown_shrinkDir_OnShow)
	UIDropDownMenu_SetWidth(scrutinizer_ConfigFrame_DropDown_shrinkDir, 70)
	local selected = 1
	if scrutinizer[playerName]["cfg"]["configframe"]["shrinkTo"] == "TOP" then
		selected = 1
	elseif scrutinizer[playerName]["cfg"]["configframe"]["shrinkTo"] == "BOTTOM" then
		selected = 2
	end
	UIDropDownMenu_SetSelectedID(scrutinizer_ConfigFrame_DropDown_shrinkDir, selected)
end

function scrutinizer_ConfigFrame_DropDown_shrinkDir_OnShow(this)
	local info= {}
	info.text = "上から"
	info.func = scrutinizer_ConfigFrame_DropDown_shrinkDir_OnClick
	UIDropDownMenu_AddButton(info)
	info = {}
	info.text = "下から"
	info.func = scrutinizer_ConfigFrame_DropDown_shrinkDir_OnClick
	UIDropDownMenu_AddButton(info)
end

function scrutinizer_ConfigFrame_DropDown_shrinkDir_OnClick(button)
	local buttonID, selected = button:GetID(), "TOP"
	UIDropDownMenu_SetSelectedID(scrutinizer_ConfigFrame_DropDown_shrinkDir, buttonID)
	if buttonID == 1 then
		selected = "TOP"
	elseif buttonID == 2 then
		selected = "BOTTOM"
	end	
	scrutinizer[playerName]["cfg"]["configframe"]["shrinkTo"] = selected
	scrutinizer_shrinkItTo(scrutinizer[playerName]["cfg"]["configframe"]["shrinkTo"])
end

function scrutinizer_SetScaleFactor(factor)
	scrutinizer_MainFrame:SetScale(factor)
	scrutinizer_DetailFrame:SetScale(factor)
	scrutinizer_ConfigFrame:SetScale(factor)
end

function scrutinizer_GetScaleFactor()
	return scrutinizer_ConfigFrame_Slider_Scale:GetValue()
end

function scrutinizer_ConfigFrame_Slider_OnLoad(this)
	scrutinizer_ConfigFrame_Slider_Scale:SetValueStepMode("FLOAT")
	scrutinizer_ConfigFrame_Slider_Scale:SetMinMaxValues(1.0, 2.0)
end

function scrutinizer_ConfigFrame_Slider_OnShow(this)
	scrutinizer_ConfigFrame_Slider_Scale:SetValue(scrutinizer[playerName]["cfg"]["configframe"]["scale"])
	getglobal(this:GetName().."Value"):SetText(scrutinizer[playerName]["cfg"]["configframe"]["scale"])
end

function scrutinizer_ConfigFrame_Slider_OnValueChanged(this)
	if addOnLoaded == false then
		return
	end
	if this.OnValueChanged then
		this:OnValueChanged()
	end
	local value = this:GetValue()
	if this:GetValueStepMode() == "FLOAT" then
		value = stringformat("%.2f", value)
	end
	getglobal(this:GetName().."Value"):SetText(value)
	scrutinizer_SetScaleFactor(value)
	scrutinizer[playerName]["cfg"]["configframe"]["scale"] = value
end

function scrutinizer_reset_mainframe_position()
	scrutinizer_MainFrame:ClearAllAnchors()
	scrutinizer_MainFrame:SetAnchor("TOPLEFT", "TOPLEFT", "UIParent", 0, 0)
end

local config_whisperTo_currentButtonID = 0
function scrutinizer_ConfigFrame_DropDown_whisperTo_OnLoad()
	UIDropDownMenu_Initialize(scrutinizer_ConfigFrame_DropDown_whisperTo, scrutinizer_ConfigFrame_DropDown_whisperTo_OnShow)
	UIDropDownMenu_SetWidth(scrutinizer_ConfigFrame_DropDown_whisperTo, 70)
	UIDropDownMenu_SetSelectedID(scrutinizer_ConfigFrame_DropDown_whisperTo, 1)
	config_whisperTo_currentButtonID = 1
end

function scrutinizer_ConfigFrame_DropDown_whisperTo_OnShow(this)
	local info= {}
	info.text = "パーティー"
	info.func = scrutinizer_ConfigFrame_DropDown_whisperTo_OnClick
	UIDropDownMenu_AddButton(info)
	info = {}
	info.text = "ギルド"
	info.func = scrutinizer_ConfigFrame_DropDown_whisperTo_OnClick
	UIDropDownMenu_AddButton(info)
	info = {}
	info.text = "個人チャンネル"
	info.func = scrutinizer_ConfigFrame_DropDown_whisperTo_OnClick
	UIDropDownMenu_AddButton(info)
	info = {}
	info.text = "チャンネル"
	info.func = scrutinizer_ConfigFrame_DropDown_whisperTo_OnClick
	UIDropDownMenu_AddButton(info)
end

function scrutinizer_ConfigFrame_DropDown_whisperTo_OnClick(button)
	local buttonID = button:GetID()
	config_whisperTo_currentButtonID = buttonID
	UIDropDownMenu_SetSelectedID(scrutinizer_ConfigFrame_DropDown_whisperTo, buttonID)
	if buttonID == 3 then
		scrutinizer_ConfigFrame_DropDown_whisperTo_ebox:Show()
		scrutinizer_ConfigFrame_DropDown_whisperTo_eboxChan:Hide()
		scrutinizer_ConfigFrame_DropDown_whisperTo_BTN:ClearAllAnchors()
		scrutinizer_ConfigFrame_DropDown_whisperTo_BTN:SetAnchor("LEFT", "RIGHT", "scrutinizer_ConfigFrame_DropDown_whisperTo", 130, 1)
	elseif buttonID == 4 then
		scrutinizer_ConfigFrame_DropDown_whisperTo_ebox:Hide()
		scrutinizer_ConfigFrame_DropDown_whisperTo_eboxChan:Show()
		scrutinizer_ConfigFrame_DropDown_whisperTo_BTN:ClearAllAnchors()
		scrutinizer_ConfigFrame_DropDown_whisperTo_BTN:SetAnchor("LEFT", "RIGHT", "scrutinizer_ConfigFrame_DropDown_whisperTo", 25, 1)
	else
		scrutinizer_ConfigFrame_DropDown_whisperTo_ebox:Hide()
		scrutinizer_ConfigFrame_DropDown_whisperTo_eboxChan:Hide()
		scrutinizer_ConfigFrame_DropDown_whisperTo_BTN:ClearAllAnchors()
		scrutinizer_ConfigFrame_DropDown_whisperTo_BTN:SetAnchor("LEFT", "RIGHT", "scrutinizer_ConfigFrame_DropDown_whisperTo", 10, 1)
	end
	scrutinizer_ConfigFrame_DropDown_whisperTo_ebox:SetText("")
	scrutinizer_ConfigFrame_DropDown_whisperTo_eboxChan:SetText("")
end

function scrutinizer_ConfigFrame_DropDown_whisperTo_BTN_OnClick()
	local eboxTxt = tostring(scrutinizer_ConfigFrame_DropDown_whisperTo_ebox:GetText())
	local eboxChanTxt = tonumber(scrutinizer_ConfigFrame_DropDown_whisperTo_eboxChan:GetText())
	local dropDownID = config_whisperTo_currentButtonID
	if dropDownID == 1 then
		scrutinizer_sendTo("PARTY", 0)
	elseif dropDownID == 2 then
		scrutinizer_sendTo("GUILD", 0)
	elseif dropDownID == 3 then
		if eboxTxt ~= "" then
			local firstChar = stringsub(eboxTxt, 1, 1)
			local restString = stringsub(eboxTxt, 2)
			firstChar = stringupper(firstChar)
			restString = stringlower(restString)
			eboxTxt = firstChar..restString
			scrutinizer_sendTo("WHISPER", eboxTxt)
		end
	elseif dropDownID == 4 then
		if eboxChanTxt ~= nil then
			if eboxChanTxt ~= "" and eboxChanTxt >= 1 and eboxChanTxt <= 9 then
				scrutinizer_sendTo("CHANNEL", eboxChanTxt)
			end
		end
	end
end

function scrutinizer_sendTo(wType, wName)
	local _, catName = scrutinizer_GetCat()
	local tbl = scrutinizer[playerName]["db"][catName]
	local txt = ""
	local sortTbl = {}
	local raidValue = 0
	for k, v in pairs(tbl) do
		if k == playerName or InPartyByName(k) == true or InRaidByName(k) == true then
			local value = v["value"]
			local lastFightValue = v["lastFightValue"]
			local startPsTime = v["startPsTime"]
			local lastPsTime = v["lastPsTime"]
			local totalPsTime = v["totalPsTime"]
			tinsert(sortTbl, {
				name = k,
				value = value,
				lastFightValue = lastFightValue,
				startPsTime = startPsTime,
				lastPsTime = lastPsTime,
				totalPsTime = totalPsTime,
			})
			raidValue = raidValue + value
		end
	end
	if #sortTbl <= 0 then
		return
	end
	tsort(sortTbl, function(a, b)
		return (a["value"] > b["value"])
	end)
	local sortedTxt = ""
	for k, v in ipairs(sortTbl) do
		sortedTxt = ""
		if v["value"] then
			local nr = tostring(k)
			local pName = tostring(stringsub(v["name"], 1, 5))
			local val = tostring(v["value"])
			local perCent = tostring(scrutinizer_DivRound(1, v["value"], raidValue))
			local Ps = tostring(scrutinizer_DivRound(0, v["value"], v["totalPsTime"]))			
			sortedTxt = stringformat("%2s%2s%5s%2s%8s(%3s%1s)%2s%5s\n", nr, "  ", pName, "  ", val, perCent, "%","  ", Ps)
			txt = txt..sortedTxt
		end
	end
	local firstLine = "\n> 成績表: "..tostring(catName).."\n"
	local attTxt = stringformat("%2s%1s%5s%8s%1s%3s%1s%5s\n", SCRUTINIZER_TXT_NO, " ", SCRUTINIZER_TXT_NAME, SCRUTINIZER_TXT_VALUE, "(", SCRUTINIZER_TXT_PERCENT, ")  ", SCRUTINIZER_TXT_PS)
	txt = stringformat("%s\n", "-----------------------------------------")..txt
	txt = attTxt..txt
	txt = firstLine..txt
	if wName == 0 then
		SendChatMessage(txt, wType)
	else
		SendChatMessage(txt, wType, 0, wName)
	end
end

--------------------------slash command functions
function scrutinizer_SlashCmdHandler(_, suf)
	if suf == "info" then
		scrutinizer_Info()
	elseif suf == "config" then
		scrutinizer_ConfigFrame:Show()
	elseif suf == "show" or suf == "zeigen" then
		scrutinizer_MainFrame:Show()
	elseif suf == "hide" or suf == "verstecken" then
		scrutinizer_MainFrame:Hide()
	else
		scrutinizer_Help()
	end
end

function scrutinizer_Info()
	DEFAULT_CHAT_FRAME:AddMessage("> "..addOnName..": V"..addOnVersion.." by "..addOnAuthor)
end

function scrutinizer_Help()
	DEFAULT_CHAT_FRAME:AddMessage("> "..addOnName..": ")
	DEFAULT_CHAT_FRAME:AddMessage(SCRUTINIZER_HELP_INFO)
	DEFAULT_CHAT_FRAME:AddMessage(SCRUTINIZER_HELP_CONFIG)
	DEFAULT_CHAT_FRAME:AddMessage(SCRUTINIZER_HELP_SHOW)
	DEFAULT_CHAT_FRAME:AddMessage(SCRUTINIZER_HELP_HIDE)
end


--------------------------tooltip functions
function scrutinizer_HandleFunctionalTooltip(this, txt)

	GameTooltip:SetOwner(this, "ANCHOR_PRESERVE", 4, 0)
	
	GameTooltip:SetText(txt, 1, 1, 1)

	if addOnLoaded == true then
		GameTooltip:SetScale(scrutinizer[playerName]["cfg"]["configframe"]["scale"])
	end
	GameTooltip:Show()
end

function scrutinizer_HandleUnitTooltip(this, name)
	local theName = name
	if theName == "" then
		return
	end
	GameTooltip:SetOwner(this, "ANCHOR_PRESERVE", 4, 0)
	local theString = ""
	local class, subclass = "", ""
	local level, sublevel = 0, 0
	for i = 1, 36, 1 do
		if theName == playerName then
			class, subclass = UnitClass("player")
			level, sublevel = UnitLevel("player")
			break
		end
		if theName == UnitName("raid"..i) then
			class, subclass = UnitClass("raid"..i)
			level, sublevel = UnitLevel("raid"..i)
			break
		end
	end
	if sublevel == 0 then
		theString = class.."("..level..")"
	else
		theString = class.."("..level..") - "..subclass.."("..sublevel..")"
	end
	GameTooltip:SetText(theName, 1, 1, 1)
	GameTooltip:AddLine(theString, 0, 0.75, 0.95)
	if addOnLoaded == true then
		GameTooltip:SetScale(scrutinizer[playerName]["cfg"]["configframe"]["scale"])
	end
	GameTooltip:Show()
end

function scrutinizer_ResetTooltipScale()
	GameTooltip:SetScale(1.0)
end

--------------------------add functions
function scrutinizer_mainframe_clean()
	scrutinizer_Main_SelfFrame_F2:SetText("")
	scrutinizer_Main_SelfFrame_F3:SetText("")
	for k = 1, 6, 1 do
		for i = 1, 3, 1 do
			_G["scrutinizer_Main_BarFrame_"..k.."F"..i]:SetText("")
		end
		_G["scrutinizer_Main_BarFrame_Bar"..k]:SetWidth(0)
		_G["scrutinizer_Main_BarFrame_Bar"..k]:SetColor(0, 0, 0, 1)
	end
	scrutinizer_Main_PartyFrame_F1:SetText("")
	scrutinizer_Main_PartyFrame_F2:SetText("")
end

function scrutinizer_ResetDB()
	local tbl = scrutinizer[playerName]["db"]
	for k, v in ipairs(cats) do
		tbl[v] = {}
	end
	scrutinizer_mainframe_clean()
	DEFAULT_CHAT_FRAME:AddMessage("> "..addOnName..": "..SCRUTINIZER_MSG_RESETDB)
end


function scrutinizer_DivRound(b, value1, value2)
	if value2 <= 0 then
		value2 = 1
	end
	if value1 == 0 then
		return 0
	end
	
	local value = 0
	
	if b == 1 then
		
		value = (100 * value1) / value2
		
		local value_d = 1 - (mceil(value) - value)
		
		if value_d >= 0.5 then
			value = mceil(value)
		elseif value_d < 0.5 then
			value = mfloor(value)
		end
		
	else
		if value2 < 1 then
			value2 = 1
		end
		value = mfloor(value1 / value2)
	end
	
	return value
end


function scrutinizer_GetCat()
	local currentCat = scrutinizer_Main_TitleFrame_Cat:GetText()
	for k, v in ipairs(cats) do
		if v == currentCat then
			return k, v
		end
	end
end

function scrutinizer_GetPage()
	return tonumber(scrutinizer_Main_PartyFrame_BarPage:GetText())
end

function scrutinizer_SetMaxPage(count)
	maxPageNum = mceil(count/6)
end

function scrutinizer_shrinkItTo(direction)
	for frm, _ in pairs(shrinkOrder[direction]) do
		_G[frm]:ClearAllAnchors()
	end
	for frm, val in pairs(shrinkOrder[direction]) do
		local point, relativePoint, relativeTo, x, y = unpack(val)
		_G[frm]:SetAnchor(point, relativePoint, relativeTo, x, y)
	end
end





--------------------------detail frame functions
function scrutinizer_DetailFrame_clean()
	for k = 1, #cats, 1 do
		_G["scrutinizer_DetailFrame_CatFrame"..k.."_Cat_Text"]:SetText("")
		_G["scrutinizer_DetailFrame_CatFrame"..k.."_TotalValue"]:SetText("")
		_G["scrutinizer_DetailFrame_CatFrame"..k.."_LastValue"]:SetText("")
		_G["scrutinizer_DetailFrame_CatFrame"..k.."_TotalPS"]:SetText("")
		_G["scrutinizer_DetailFrame_CatFrame"..k.."_LastPS"]:SetText("")
	end
	for k = 1, 25, 1 do
		for i = 1, 11, 1 do
			_G["scrutinizer_DetailFrame_Body_Row"..k.."_F"..i]:SetText("")
		end
	end
end

function scrutinizer_DoDetail(currentName)
	local name = currentName
	if name == "" then
		return
	end
	
	local theString = ""
	local class, subclass = "", ""
	local level, sublevel = 0, 0
	for i = 1, 36, 1 do
		if name == playerName then
			class, subclass = UnitClass("player")
			level, sublevel = UnitLevel("player")
			break
		end
		if name == UnitName("raid"..i) then
			class, subclass = UnitClass("raid"..i)
			level, sublevel = UnitLevel("raid"..i)
			break
		end
	end
	if sublevel == 0 then
		theString = class.."("..level..")"
	else
		theString = class.."("..level..") - "..subclass.."("..sublevel..")"
	end
	
	
	
	scrutinizer_DetailFrame_TitleFrame_Name:SetText(name)
	scrutinizer_DetailFrame_TitleFrame_NameInfo:SetText(theString)
	
	
	scrutinizer_DetailFrame_clean()


	--set currentcat to skilltable
	local currentCatID, currentCatName = scrutinizer_GetCat()	
	--print ps numbers
	for k = 1, #cats, 1 do
		_G["scrutinizer_DetailFrame_CatFrame"..k.."_Cat_Text"]:SetText(cats[k])
		if currentCatID == k then
			_G["scrutinizer_DetailFrame_BTN_Cat"..k]:LockHighlight()
		else
			_G["scrutinizer_DetailFrame_BTN_Cat"..k]:UnlockHighlight()
		end
		scrutinizer_Detail_PS(k, name)
	end
	
	--print skills
	scrutinizer_Detail_Skills(currentCatName, name)
	
	scrutinizer_DetailFrame:Show()
end

function scrutinizer_Detail_PS(catID, name)
	if not scrutinizer[playerName]["db"][cats[catID]][name] then
		return
	end
	local tbl = scrutinizer[playerName]["db"][cats[catID]][name]
	
	_G["scrutinizer_DetailFrame_CatFrame"..catID.."_TotalValue"]:SetText(
		SCRUTINIZER_TXT_VALUE..": "..tbl["value"]
	)
	
	_G["scrutinizer_DetailFrame_CatFrame"..catID.."_LastValue"]:SetText(
		SCRUTINIZER_TXT_VALUELASTFIGHT..": "..tbl["lastFightValue"]
	)
	
	_G["scrutinizer_DetailFrame_CatFrame"..catID.."_TotalPS"]:SetText(
		SCRUTINIZER_TXT_PS..": "..scrutinizer_DivRound(0, tbl["value"], tbl["totalPsTime"])
	)
	
	_G["scrutinizer_DetailFrame_CatFrame"..catID.."_LastPS"]:SetText(
		SCRUTINIZER_TXT_PSLASTFIGHT..": "..scrutinizer_DivRound(0, tbl["lastFightValue"], (tbl["lastPsTime"] - tbl["startPsTime"]))
	)
	
end

function scrutinizer_Detail_Skills(currentCatName, name)
	if not scrutinizer[playerName]["db"][currentCatName][name] then
		return
	end
	
	local tbl = scrutinizer[playerName]["db"][currentCatName][name]["skills"]
	local sortTbl = {}
	local skillTypeCounter = 0
	local skillTotalValue = 0
	
	for skillName, skillInfo in pairs(tbl) do
		skillTypeCounter = 0
		skillTypeCounter = skillInfo["normal"] + skillInfo["critical"] + skillInfo["dodge"] + skillInfo["parry"] + skillInfo["miss"] + skillInfo["resist"] + skillInfo["double"]
		skillTotalValue = skillTotalValue + skillInfo["value"]
		tinsert(sortTbl, {
			skill = skillName,
			value = skillInfo["value"],
			normal = skillInfo["normal"],
			critical = skillInfo["critical"],
			dodge = skillInfo["dodge"],
			parry = skillInfo["parry"],
			miss = skillInfo["miss"],
			resist = skillInfo["resist"],
			double = skillInfo["double"],
			counter = skillTypeCounter,
		})
	end
	
	--sort by value
	tsort(sortTbl, function(a, b)
		return (a["value"] > b["value"])
	end)
	
	local row = 1
	local maxRowNum = 25
	--print body
	for k, v in ipairs(sortTbl) do
		local realSkillName = v["skill"]
		if realSkillName == "" then
			realSkillName = SCRUTINIZER_TXT_NOSKILLNAME
		elseif realSkillName == "ATTACK" then
			realSkillName = SCRUTINIZER_TXT_ATTACK
		end
		_G["scrutinizer_DetailFrame_Body_Row"..row.."_F1"]:SetText(realSkillName)
		_G["scrutinizer_DetailFrame_Body_Row"..row.."_F2"]:SetText(v["value"].." ("..scrutinizer_DivRound(1, v["value"], skillTotalValue).."%)")
		_G["scrutinizer_DetailFrame_Body_Row"..row.."_F3"]:SetText(v["counter"])
		_G["scrutinizer_DetailFrame_Body_Row"..row.."_F4"]:SetText(scrutinizer_DivRound(0, v["value"], v["counter"]))
		
		_G["scrutinizer_DetailFrame_Body_Row"..row.."_F5"]:SetText(scrutinizer_DivRound(1, v["normal"], v["counter"]).."%")
		_G["scrutinizer_DetailFrame_Body_Row"..row.."_F6"]:SetText(scrutinizer_DivRound(1, v["critical"], v["counter"]).."%")
		_G["scrutinizer_DetailFrame_Body_Row"..row.."_F7"]:SetText(scrutinizer_DivRound(1, v["double"], v["counter"]).."%")
		_G["scrutinizer_DetailFrame_Body_Row"..row.."_F8"]:SetText(scrutinizer_DivRound(1, v["miss"], v["counter"]).."%")
		_G["scrutinizer_DetailFrame_Body_Row"..row.."_F9"]:SetText(scrutinizer_DivRound(1, v["resist"], v["counter"]).."%")
		_G["scrutinizer_DetailFrame_Body_Row"..row.."_F10"]:SetText(scrutinizer_DivRound(1, v["dodge"], v["counter"]).."%")
		_G["scrutinizer_DetailFrame_Body_Row"..row.."_F11"]:SetText(scrutinizer_DivRound(1, v["parry"], v["counter"]).."%")
		
		if row == #sortTbl then
			break
		end
		
		if row == maxRowNum then
			break
		end
		
		row = row + 1
	end
	
end


function scrutinizer_Highlight_Detail_Cat_Buttons(this, catNameToChangeTo)
	this:LockHighlight()
	local theName = scrutinizer_DetailFrame_TitleFrame_Name:GetText()
	
	local currentCatID = 1
	
	scrutinizer_DetailFrame_clean()
	for k, v in ipairs(cats) do
		if v == catNameToChangeTo then
			currentCatID = k
			break
		end
	end

	--print ps numbers
	for k = 1, #cats, 1 do
		_G["scrutinizer_DetailFrame_CatFrame"..k.."_Cat_Text"]:SetText(cats[k])
		if currentCatID == k then
			_G["scrutinizer_DetailFrame_BTN_Cat"..k]:LockHighlight()
		else
			_G["scrutinizer_DetailFrame_BTN_Cat"..k]:UnlockHighlight()
		end
		scrutinizer_Detail_PS(k, theName)
	end
	
	--print skills
	scrutinizer_Detail_Skills(catNameToChangeTo, theName)
	
end



function scrutinizer_DetailFrame_Body_OnLoad()
	--clear all anchors
	_G["scrutinizer_DetailFrame_Body_Att"]:ClearAllAnchors()
	for i = 1, 11, 1 do
		_G["scrutinizer_DetailFrame_Body_Att_F"..i]:ClearAllAnchors()
	end
	for k = 1, 25, 1 do
		for i = 1, 11, 1 do
			_G["scrutinizer_DetailFrame_Body_Row"..k.."_F"..i]:ClearAllAnchors()
		end
		_G["scrutinizer_DetailFrame_Body_Row"..k]:ClearAllAnchors()
	end
	
	--set att anchors
	_G["scrutinizer_DetailFrame_Body_Att_F1"]:SetAnchor("TOPLEFT", "TOPLEFT", "scrutinizer_DetailFrame_BodyFrame", 15, 15)
	_G["scrutinizer_DetailFrame_Body_Att_F2"]:SetAnchor("TOPLEFT", "TOPLEFT", "scrutinizer_DetailFrame_BodyFrame", 130, 15)
	_G["scrutinizer_DetailFrame_Body_Att_F3"]:SetAnchor("TOPLEFT", "TOPLEFT", "scrutinizer_DetailFrame_BodyFrame", 250, 15)
	_G["scrutinizer_DetailFrame_Body_Att_F4"]:SetAnchor("TOPLEFT", "TOPLEFT", "scrutinizer_DetailFrame_BodyFrame", 320, 15)
	
	_G["scrutinizer_DetailFrame_Body_Att_F5"]:SetAnchor("TOPLEFT", "TOPLEFT", "scrutinizer_DetailFrame_BodyFrame", 390, 15)
	_G["scrutinizer_DetailFrame_Body_Att_F6"]:SetAnchor("TOPLEFT", "TOPLEFT", "scrutinizer_DetailFrame_BodyFrame", 450, 15)
	_G["scrutinizer_DetailFrame_Body_Att_F7"]:SetAnchor("TOPLEFT", "TOPLEFT", "scrutinizer_DetailFrame_BodyFrame", 510, 15)
	_G["scrutinizer_DetailFrame_Body_Att_F8"]:SetAnchor("TOPLEFT", "TOPLEFT", "scrutinizer_DetailFrame_BodyFrame", 570, 15)
	_G["scrutinizer_DetailFrame_Body_Att_F9"]:SetAnchor("TOPLEFT", "TOPLEFT", "scrutinizer_DetailFrame_BodyFrame", 630, 15)
	_G["scrutinizer_DetailFrame_Body_Att_F10"]:SetAnchor("TOPLEFT", "TOPLEFT", "scrutinizer_DetailFrame_BodyFrame", 690, 15)
	_G["scrutinizer_DetailFrame_Body_Att_F11"]:SetAnchor("TOPLEFT", "TOPLEFT", "scrutinizer_DetailFrame_BodyFrame", 750, 15)
	
	--set first row fontstrings to att fontstrings anchors
	for i = 1, 11, 1 do
		_G["scrutinizer_DetailFrame_Body_Row1_F"..i]:SetAnchor("TOPLEFT", "BOTTOMLEFT", _G["scrutinizer_DetailFrame_Body_Att_F"..i], 0, 0)
	end
	
	--set all other fontstring anchors
	for k = 2, 25, 1 do
		for i = 1, 11, 1 do
			_G["scrutinizer_DetailFrame_Body_Row"..k.."_F"..i]:SetAnchor("TOPLEFT", "BOTTOMLEFT", _G["scrutinizer_DetailFrame_Body_Row"..(k-1).."_F"..i], 0, 0)
		end
	end
	
	--set each frame
	for k = 1, 25, 1 do
		_G["scrutinizer_DetailFrame_Body_Row"..k]:SetAnchor("TOPLEFT", "TOPLEFT", _G["scrutinizer_DetailFrame_Body_Row"..k.."_F1"], 0, 0)
	end
	
end


function scrutinizer_DetailFrame_Body_Rows_OnEnter(this)
	local txt = _G[this:GetName().."_F1"]:GetText()
	if txt == "" then
		return
	end
	
	GameTooltip:SetOwner(this, "ANCHOR_PRESERVE", 4, 0)
	GameTooltip:SetText(txt, 1, 1, 1)
	GameTooltip:SetScale(scrutinizer[playerName]["cfg"]["configframe"]["scale"])
	GameTooltip:Show()
end

